@@ -73,22 +73,9 @@ LinternaMagica.prototype.create_youtube_links = function(fmt, fmt_url_map)
 	    // http://en.wikipedia.org/wiki/YouTube#Quality_and_codecs
 
 	    var label="";
+	    var more_info = "";
 	    var fmt_id = link_data[0];
 
-	    switch (fmt_id)
-	    {
-	    case '82':
-	    case '83':
-	    case '84':
-	    case '85':
-	    case '100':
-	    case '101':
-	    case '102':
-		label += "3D ";
-		break;
-	    default:
-		"";
-	    }
 	    // Set container
 	    switch (fmt_id)
 	    {
@@ -124,11 +111,26 @@ LinternaMagica.prototype.create_youtube_links = function(fmt, fmt_url_map)
 		label += this._("Unkown container");
 	    }
 
+	    switch (fmt_id)
+	    {
+	    case '82':
+	    case '83':
+	    case '84':
+	    case '85':
+	    case '100':
+	    case '101':
+	    case '102':
+		label += " 3D";
+		break;
+	    default:
+		"";
+	    }
+
 	    // Set video and audio encodings
 	    switch (fmt_id)
 	    {
 	    case '5':
-		label += " Sorenson H.263, MP3";
+		more_info += "Sorenson H.263, MP3";
 		break;
 	    case '18':
 	    case '22':
@@ -140,7 +142,7 @@ LinternaMagica.prototype.create_youtube_links = function(fmt, fmt_url_map)
 	    case '83':
 	    case '84':
 	    case '85':
-		label += " MPEG-4 AVC (H.264), AAC";
+		more_info += "MPEG-4 AVC (H.264), AAC";
 		break;
 	    case '43':
 	    case '44':
@@ -149,18 +151,18 @@ LinternaMagica.prototype.create_youtube_links = function(fmt, fmt_url_map)
 	    case '100':
 	    case '101':
 	    case '102':
-		label += " VP8, Vorbis";
+		more_info += "VP8, Vorbis";
 		break;
 	    case '13':
 	    case '17':
-		label += " MPEG-4 Visual, AAC";
+		nore_info += "MPEG-4 Visual, AAC";
 		break;
 	    default:
-		label += " " + this._("Unkown encoding");
+		more_info += " " + this._("Unkown encoding");
 	    }
 
-	    link.label  = link_data[1] + " " +label;
-
+	    link.label  = link_data[1].split(/x/)[1] + "p " +label;
+	    link.more_info = link_data[1] + " " +label+ " " + more_info;
 	    if (!fmt_url_map[fmt_id])
 	    {
 		this.log("LinternaMagica.create_youtube_links:\n"+
@@ -169,7 +171,7 @@ LinternaMagica.prototype.create_youtube_links = function(fmt, fmt_url_map)
 		continue;
 	    }
 
-	    link.url = fmt_url_map[fmt_id].replace(/\\u0026/g,"&");
+	    link.url = fmt_url_map[fmt_id];
 
 	    this.log("LinternaMagica.create_youtube_links:\n"+
 		     "Extracted link  : "+link.url,4);
@@ -183,7 +185,7 @@ LinternaMagica.prototype.create_youtube_links = function(fmt, fmt_url_map)
     return null;
 }
 
-// Detect youtube flash upgrade warning This is called withing
+// Detect youtube flash upgrade warning. This is called withing
 // setInterval. It is needed because when the elements with the
 // warning are inserted all our data that has been added before that
 // is removed.
@@ -191,10 +193,30 @@ LinternaMagica.prototype.detect_youtube_flash_upgrade = function(object_data)
 {
     this.youtube_flash_upgrade_counter++;
 
-    // With default timeout 500mS this will be 3 sec. Stop checking and insert.    
+    // Fancy flash upgrade message. The element with id flash-upgrade
+    // might be obsolete now.
+    var watch_player = document.getElementById("watch-player");
+    var fancy_alert = null;
+
+    if(watch_player && watch_player.hasAttribute("class") &&
+       /flash-player/i.test(watch_player.getAttribute("class")))
+    {
+	var alert = watch_player.querySelector(".yt-alert-message");
+
+	if (alert && /flash player/i.test(alert.textContent))
+	{
+	    fancy_alert = true;
+	}
+	
+    }
+    
+
+    // With default timeout 500mS this will be 6 sec. Stop checking and insert.    
     // Might be flashblock
     if (document.getElementById("flash-upgrade") ||
-	this.youtube_flash_upgrade_counter >= 6 )
+	document.getElementById("movie_player-html5") ||
+	fancy_alert ||
+	this.youtube_flash_upgrade_counter >= 12 )
     {
 	clearInterval(this.youtube_flash_upgrade_timeout);
 
@@ -229,13 +251,14 @@ LinternaMagica.prototype.extract_youtube_fmt_url_map = function(data)
 	// Hash with keys fmt_ids and values video URLs
 	var map = new Object();
 
-	fmt = unescape(fmt[fmt.length-1].replace(/\\\//g, "/"));
+	fmt = fmt[fmt.length-1].replace(/\\\//g, "/");
 	fmt = fmt.split(/,/);
 
 	var links = 0;
 
 	for (var url=0; url<fmt.length; url++) 
 	{
+	    // Usually the links have the following pattern
 	    // (itag=fmt_id)*url=URL&type=video/...&(itag=fmt_id)*
 	    var link = fmt[url].match(/url=([^&]+)/);
 	    var fmt_id = fmt[url].match(/itag=([0-9]+)/);
@@ -243,7 +266,10 @@ LinternaMagica.prototype.extract_youtube_fmt_url_map = function(data)
 	    if (fmt_id && link)
 	    {
 		links++;
-		map[fmt_id[fmt_id.length-1]] =  unescape(link[link.length-1]);
+		link = unescape(link[link.length-1]);
+		link = link.split(/\\u0026/)[0];
+
+		map[fmt_id[fmt_id.length-1]] =  link;
 	    }
 	}
 
@@ -269,22 +295,22 @@ LinternaMagica.prototype.sites["youtube-nocookie.com"] = "youtube.com";
 LinternaMagica.prototype.sites["youtube.com"].flash_plugin_installed =
 function()
 {
-    var site_html5_player = this.find_site_html5_player_wrapper(document);
-
-    // If there is html5 player and flash plugin is installed no SWF
-    // object will be created. We must examine scripts.
-    if (site_html5_player)
-    {
-	return this.sites.__no_flash_plugin_installed.apply(this, [arguments]);
-    }
-
-    return true;
+    // We must examine scripts because searching for links in DOM
+    // objects in YouTube bloats GNU IceCat. The other option is to
+    // use the matched video_id and XHR the page again from where to
+    // get the links. That is not acceptable. No need to duplicate the
+    // code in __no_flash_plugin_installed().
+    return this.sites.__no_flash_plugin_installed.apply(this, [arguments]);
 }
 
-LinternaMagica.prototype.sites["youtube.com"].set_cookies_domain =
+LinternaMagica.prototype.sites["youtube.com"].skip_script_processing =
 function()
 {
-    return ".youtube.com";
+    // See bug #108013:
+    // https://savannah.nongnu.org/support/index.php?108013
+    // Do not skip script processing in youtube. The default function
+    // might skip scripts in some browsers.
+    return true;
 }
 
 LinternaMagica.prototype.sites["youtube.com"].skip_link_extraction = function()
@@ -316,12 +342,17 @@ function()
     var width = data.match(/\"width\"\:\s*\"([0-9]+)\"/);
 
     this.extract_video_id_data = data;
-    var video_id = this.extract_video_id();
+
+    var fmt = this.extract_youtube_fmt_parameter(data);
+    var maps = this.extract_youtube_fmt_url_map(data);
+
+    var hd_links = this.create_youtube_links(fmt, maps);
+    var link = (hd_links && hd_links.length) ? hd_links : null;
 
     var embed_id = data.match(/\"id\"\:\s*\"([a-zA-Z0-9_\-]+)\"/);
 
-    // We do not have any links!
-    if (!video_id)
+    // We do not have any links! Give up.
+    if (!link)
     {
 	return null;
     }
@@ -362,7 +393,8 @@ function()
 
     object_data.width= width;
     object_data.height= height;
-    object_data.video_id=video_id;
+    object_data.link = hd_links ? hd_links[hd_links.length-1].url : null;
+    object_data.hd_links = link;
 
     if (embed_id)
     {
@@ -389,6 +421,7 @@ function()
 
 	object_data.parent = document.getElementById("watch-player");
     }
+
     object_data.linterna_magica_id = linterna_magica_id;
 
     return object_data;
@@ -404,91 +437,14 @@ function()
     return false;
 }
 
-// Just return true or false. The function must be defined, so the
-// default code will not be executed. YouTube's object is created
-// after XHR.
 LinternaMagica.prototype.sites["youtube.com"].
     replace_extracted_object_from_script = 
-function()
-{
-    return false;
-}
-
-LinternaMagica.prototype.sites["youtube.com"].prepare_xhr =
 function(object_data)
 {
-    var result = new Object();
-
-    var location_href = window.location.href;
-
-    var uri_args = null;
-    // Some clips require &skipcontrinter=1. Other might require
-    // something else.
-    if (/&/i.test(location_href))
-    {
-	uri_args = location_href.split(/&/);
-	// This is the host and path (http://...). We do not need
-	// it.
-	delete uri_args[0];
-	uri_args = uri_args.join("&");
-    }
-
-    result.address = "/watch?v="+object_data.video_id+
-	(uri_args ? ("&"+uri_args) : "");
-
-    // Remove cookies and fetch page again. See "A note on
-    // cookies".
-    this.extract_cookies();
-    this.expire_cookies();
-
-    return result;
-}
-
-LinternaMagica.prototype.sites["youtube.com"].process_xhr_response =
-function(args)
-{
-    var client = args.client;
-    var object_data = args.object_data;
-
-    var fmt = this.extract_youtube_fmt_parameter(client.responseText);
-    var maps = this.extract_youtube_fmt_url_map(client.responseText);
-
-    var hd_links = this.create_youtube_links(fmt, maps);
-
-    object_data.link = hd_links ? hd_links[hd_links.length-1].url : null;
-    object_data.hd_links = (hd_links && hd_links.length) ? hd_links : null;
-
-    // See "A note on cookies"
-    if (/restore/i.test(this.process_cookies))
-    {
-	this.restore_cookies();
-    }
-
-    return object_data;
-}
-
-LinternaMagica.prototype.sites["youtube.com"].
-    process_duplicate_object_before_xhr =
-function(object_data)
-{
-    this.log("LinternaMagica.sites.process_duplicate_object_before_xhr:\n"+
-	     "Removing/hiding duplicate object ",1);
-
-    this.hide_flash_video_object(object_data.linterna_magica_id,
-				 object_data.parent);
-
-    return false;
-}
-
-LinternaMagica.prototype.sites["youtube.com"].insert_object_after_xhr =
-function(object_data)
-{
-    // Just return true and let the default code do its job. A special
-    // attention is needed when no plugin is installed.
-    if (this.plugin_is_installed)
-    {
-	return true;
-    }
+    // This function used to exit if flash plugin was installed,
+    // because no workarounds were needed. Now Epiphany requires this
+    // delay, because it prevents LM to be inserted before the HTML5
+    // player, in which case they are both playing and overlapping.
 
     if (!this.youtube_flash_upgrade_timeout)
     {
@@ -505,7 +461,8 @@ function(object_data)
     return false;
 }
 
-LinternaMagica.prototype.sites["youtube.com"].css_fixes = function(object_data)
+LinternaMagica.prototype.sites["youtube.com"].css_fixes =
+function(object_data)
 {
     if (document.getElementById("playnav-playview"))
     {
@@ -561,7 +518,7 @@ LinternaMagica.prototype.sites["youtube.com"].css_fixes = function(object_data)
 	}
     }
 
-     // Bug #33504 https://savannah.nongnu.org/bugs/?33504
+    // Bug #33504 https://savannah.nongnu.org/bugs/?33504
     object_data.parent.style.setProperty("overflow", "visible", "important");
 
     var site_html5_player = 
@@ -577,7 +534,80 @@ LinternaMagica.prototype.sites["youtube.com"].css_fixes = function(object_data)
 						 "50px", "important");
 	
     }
+
+    // Sometimes the toggle plugin button that should be bellow the
+    // HTML5/Flash player shows on top. The following tries to fix
+    // this.  Most of the time the bug occurs in Epiphany.s
+
+    var id = object_data.linterna_magica_id;
+    var toggle_plugin = document.getElementById("linterna-magica-toggle-plugin-"+id);
     
+    if (toggle_plugin)
+    {
+	var lm = document.getElementById("linterna-magica-"+id);
+	var h_lm = parseInt(lm.style.getPropertyValue("height"));
+
+	toggle_plugin = toggle_plugin.parentNode;
+	var display = toggle_plugin.style.getPropertyValue("display");
+	var top = toggle_plugin.offsetTop;
+
+	if (!display && !top)
+	{
+	    toggle_plugin.style.setProperty("top", 
+					    (h_lm +30)+"px",
+					    "important");
+	}
+    }
+
+    // When YouTube's HTML5 player detects that the browser does not
+    // have support for any clip, a warning div is displayed. It must
+    // be hidden or it displaces Linterna Magica.
+
+    // The <video/> element is missing because of the
+    // error. this.find_site_html5_wrapper() will not
+    // work. Implementing custom_html5_player_finder() is not so good
+    // idea.
+    var html5_wrapper = document.getElementById("movie_player-html5");
+
+    if (html5_wrapper)
+    {
+	var html5_warning = html5_wrapper.querySelector(".video-fallback");
+
+	// The clips that are displayed as unavailable have different
+	// class. Sometimes?
+	//
+	// Fixes bug #35992 https://savannah.nongnu.org/bugs/?35992
+	if (!html5_warning)
+	{
+	    html5_warning =
+		html5_wrapper.querySelector(".html5-video-fallback");
+	}
+
+	if (html5_warning &&
+	    !/none/i.test(html5_warning.style.getPropertyValue('display')))
+	{
+	    // Must hide the entire HTML5 wrapper, so Linterna Magica
+	    // will not be displaced.
+	    html5_wrapper.style.setProperty('display', "none", "important");
+
+	    // Hide site controls and video container. Should not
+	    // overlap with Linterna Magica now. Fixes the overlapping
+	    // of HTML5 player and LM.
+	    var controls = html5_wrapper.querySelector(".html5-video-controls");
+	    var container = html5_wrapper.querySelector(".html5-video-container");
+
+	    if (controls)
+	    {
+		controls.style.setProperty("display", "none", "important");
+	    }
+
+	    if (container)
+	    {
+		container.style.setProperty("display", "none", "important");
+	    }
+
+	}
+    }
 
     return false;
 }
